# 开发规范

## 代码规范

- 尽量使用[IntelliJ IDEA](https://www.jetbrains.com/idea/)开发
- 尽可能遵守[阿里巴巴Java开发规范](https://github.com/alibaba/p3c/raw/master/%E9%98%BF%E9%87%8C%E5%B7%B4%E5%B7%B4Java%E5%BC%80%E5%8F%91%E6%89%8B%E5%86%8C%EF%BC%88%E5%8D%8E%E5%B1%B1%E7%89%88%EF%BC%89.pdf)
- 使用IDEA开发时，建议使用[阿里巴巴代码规范插件](https://github.com/alibaba/p3c/blob/master/idea-plugin/README_cn.md)

## POM

- 项目所需包版本号在parent中的```dependencyManagement```定，子模块只需引用即可

## 实体类

- 实体类需要实现在```cn.edu.sysu.workflow.common.entity```包内
- 数据库实体类需要继承```BooPagedQuery```类，便于分页查询
- 数据库实体类需要生成```serialVersionUID```
- 数据库实体类需要创建```PREFIX```
- 数据库实体类属性需要添加易读注释
- 数据库实体类只需实现无参构造函数，逻辑仅为```super()```
- 数据库实体类不可使用简单属性，需要使用包装类
- 异常实体类需要实现在```cn.edu.sysu.workflow.common.entity.exception```包内
- 不要直接抛出```RuntimeException```，而是自定义异常抛出

## 枚举类型

- Type型参数需要使用枚举类型实现
- 枚举类型需要实现在```cn.edu.sysu.workflow.common.enums```包内
- 枚举类型字段添加合适的注释

## 工具类

- 公用的工具类需要实现在```cn.edu.sysu.workflow.common.util```包内
- 工具类需要标识为final

## DAO层

- DAO层逻辑代码可能比较复杂，因此实现为接口类实现类分离的抽象方式，便于主导者定义接口而不需关心底层实现，DAO层开发者专注于底层实现而不需要关心上层逻辑
- ```@Repository```注解放在实现类上
- 接口类需要写详细的javadoc注释，传入类时，需要表明可传入的类属性
- 实现类参考Skye已有实现代码
- 插入数据方法命名为save()，更新数据方法命名为update()，读取数据方法命名为findXXX()，删除数据方法命名为deleteXXX()
- 捕获异常log打印日志，并抛出```DAOException```交由Service层处理
- 查询语句处理```EmptyResultDataAccessException```异常只需返回null即可

## Service层

- Service层逻辑代码可能比较复杂，因此实现为接口类实现类分离的抽象方式，便于主导者定义接口而不需关心底层实现，Service层开发者专注于底层实现而不需要关心上层逻辑
- ```@Service```注解放在实现类上
- 接口类需要写详细的javadoc注释，传入类时，需要表明可传入的类属性
- 实现类参考Skye已有实现代码
- 捕获异常log打印日志，并抛出```ServiceFailureException```交由Controller统一异常处理

## Controller层

- 全部实现为Restful接口
- 所有的异常处理交由```GlobalExceptionHandler```（Controller统一异常处理）处理

## 配置文件

- 不同环境需使用不同配置文件，运行前检查好

## 单元测试

- 尽可能覆盖

# 代码仓库规范

## 分支

- 分支类型：d
- 版本号：大版本.小版本.bug修复版本
- 开发分支名称模板：dev/开发者名称/版本号
- 发布分支名称模板：release/版本号

## 管理

- 不可所有人直接merge，除非项目只有一人管理，否则至少有一名reviewer